@@ -60,7 +60,7 @@ namespace ISMRMRD
   {
 
   public:
-    /** Default construtor */
+    /** Default constructor */
     MetaValue()
     {
       set(0L);
@@ -166,7 +166,7 @@ namespace ISMRMRD
   {
     typedef std::map< std::string, std::vector<MetaValue> > map_t;
 
-    friend void serialize(MetaContainer& h, std::ostream& o);
+    friend void EXPORTISMRMRD serialize(MetaContainer& h, std::ostream& o);
 
   public:
     MetaContainer()
@@ -231,7 +231,7 @@ namespace ISMRMRD
     {
       map_t::const_iterator it = map_.find(std::string(name));
       if (it == map_.end()) {
-	throw std::runtime_error("Attempting to access unkown parameter");
+	throw std::runtime_error("Attempting to access unknown parameter");
       }
       if (index >= it->second.size()) {
 	throw std::runtime_error("Attempting to access indexed value out of bounds");
