@@ -44,6 +44,13 @@ namespace ISMRMRD
       value_ = v;      
     }
 
+	const Optional& operator=(const Optional& o) {
+		present_ = o.present_;
+		if (present_)
+			value_ = o.value_;	
+		return *this;
+	}
+
     const Optional& operator=(const T& v) {
       present_ = true;
       value_ = v;
@@ -289,12 +296,21 @@ namespace ISMRMRD
     Optional<std::string> interleavingDimension;
   };
 
+  enum class TrajectoryType {
+      CARTESIAN,
+      EPI,
+      RADIAL,
+      GOLDENANGLE,
+      SPIRAL,
+      OTHER
+  };
+
   struct Encoding
   {
     EncodingSpace encodedSpace;
     EncodingSpace reconSpace;
     EncodingLimits encodingLimits;
-    std::string trajectory;
+    TrajectoryType trajectory;
     Optional<TrajectoryDescription> trajectoryDescription;
     Optional<ParallelImaging> parallelImaging;
     Optional<long> echoTrainLength;
@@ -310,6 +326,22 @@ namespace ISMRMRD
     Optional<std::vector<float> > echo_spacing;
   };
 
+  enum class WaveformType {
+      ECG,
+      PULSE,
+      RESPIRATORY,
+      TRIGGER,
+      GRADIENTWAVEFORM,
+      OTHER
+  };
+
+
+  struct WaveformInformation{
+      std::string waveformName;
+      WaveformType waveformType;
+      Optional<UserParameters> userParameters;
+  };
+
   struct IsmrmrdHeader
   {
     Optional<long> version;
@@ -320,7 +352,8 @@ namespace ISMRMRD
     ExperimentalConditions experimentalConditions;
     std::vector<Encoding> encoding;
     Optional<SequenceParameters> sequenceParameters;
-    Optional<UserParameters> userParameters;    
+    Optional<UserParameters> userParameters;
+    std::vector<WaveformInformation> waveformInformation;
   };
 
 
