@@ -11,6 +11,7 @@
 #endif /* __cplusplus */
 
 #include <hdf5.h>
+#include <ismrmrd/waveform.h>
 #include "ismrmrd/dataset.h"
 
 #ifdef __cplusplus
@@ -152,13 +153,19 @@ static int delete_var(const ISMRMRD_Dataset *dset, const char *var) {
 /*********************************************/
 /* Private (Static) Functions for HDF5 Types */
 /*********************************************/
-typedef struct HDF5_Acquisition
+typedef struct HDF5_Acquisiton
 {
     ISMRMRD_AcquisitionHeader head;
     hvl_t traj;
     hvl_t data;
 } HDF5_Acquisition;
 
+typedef struct HDF5_Waveform
+{
+    ISMRMRD_WaveformHeader head;
+    hvl_t data;
+} HDF5_Waveform;
+
 static hid_t get_hdf5type_uint16(void) {
     hid_t datatype = H5Tcopy(H5T_NATIVE_UINT16);
     return datatype;
@@ -260,6 +267,8 @@ static hid_t get_hdf5type_encoding(void) {
     return datatype;
 }
 
+
+
 static hid_t get_hdf5type_acquisitionheader(void) {
     hid_t datatype;
     herr_t h5status;
@@ -424,7 +433,58 @@ static hid_t get_hdf5type_image_attribute_string(void) {
     }
     return datatype;
 }
-    
+
+
+
+static hid_t get_hdf5type_waveformheader(void) {
+    hid_t datatype;
+    herr_t h5status;
+
+	datatype = H5Tcreate(H5T_COMPOUND, sizeof(ISMRMRD_WaveformHeader));
+    h5status = H5Tinsert(datatype, "version", HOFFSET(ISMRMRD_WaveformHeader, version), H5T_NATIVE_UINT16);
+    h5status = H5Tinsert(datatype, "flags", HOFFSET(ISMRMRD_WaveformHeader, flags), H5T_NATIVE_UINT64);
+    h5status = H5Tinsert(datatype, "measurement_uid", HOFFSET(ISMRMRD_WaveformHeader,  measurement_uid), H5T_NATIVE_UINT32);
+    h5status = H5Tinsert(datatype, "scan_counter", HOFFSET(ISMRMRD_WaveformHeader, scan_counter), H5T_NATIVE_UINT32);
+    h5status = H5Tinsert(datatype, "time_stamp", HOFFSET(ISMRMRD_WaveformHeader, time_stamp ), H5T_NATIVE_UINT32);
+    h5status = H5Tinsert(datatype, "number_of_samples", HOFFSET(ISMRMRD_WaveformHeader, number_of_samples), H5T_NATIVE_UINT16);
+    h5status = H5Tinsert(datatype, "channels", HOFFSET(ISMRMRD_WaveformHeader, channels), H5T_NATIVE_UINT16);
+    h5status = H5Tinsert(datatype, "sample_time_us", HOFFSET(ISMRMRD_WaveformHeader, sample_time_us), H5T_NATIVE_FLOAT);
+	h5status = H5Tinsert(datatype, "waveform_id", HOFFSET(ISMRMRD_WaveformHeader, waveform_id), H5T_NATIVE_UINT16);
+
+      /* Clean up */
+    if (h5status < 0) {
+        ISMRMRD_PUSH_ERR(ISMRMRD_FILEERROR, "Failed get waveform header data type");
+    }
+
+    return  datatype;
+
+}
+
+static hid_t get_hdf5type_waveform(void) {
+    hid_t datatype, vartype, vlvartype;
+    herr_t h5status;
+
+    datatype = H5Tcreate(H5T_COMPOUND, sizeof(HDF5_Waveform));
+    vartype = get_hdf5type_waveformheader();
+    h5status = H5Tinsert(datatype, "head", HOFFSET(HDF5_Waveform, head), vartype);
+	if (h5status < 0) {
+		ISMRMRD_PUSH_ERR(ISMRMRD_FILEERROR, "Failed get waveform header data type");
+	}
+    H5Tclose(vartype);
+	vartype = get_hdf5type_uint32();
+    vlvartype = H5Tvlen_create(vartype);
+    h5status = H5Tinsert(datatype, "data", HOFFSET(HDF5_Waveform, data), vlvartype);
+    H5Tclose(vartype);
+    H5Tclose(vlvartype);
+
+    /* Store acquisition data as an array of floats */
+
+    if (h5status < 0) {
+        ISMRMRD_PUSH_ERR(ISMRMRD_FILEERROR, "Failed get waveform data type");
+    }
+
+    return datatype;
+}
 static hid_t get_hdf5type_ndarray(uint16_t data_type) {
     
     hid_t hdfdatatype = -1;
@@ -657,6 +717,12 @@ static int append_element(const ISMRMRD_Dataset * dset, const char * path,
     offset[0] = hdfdims[0]-1;
     filespace = H5Dget_space(dataset);
     h5status  = H5Sselect_hyperslab (filespace, H5S_SELECT_SET, offset, NULL, ext_dims, NULL);
+	
+	if (h5status < 0) {
+	
+		H5Ewalk2(H5E_DEFAULT, H5E_WALK_UPWARD, walk_hdf5_errors, NULL);
+		return ISMRMRD_PUSH_ERR(ISMRMRD_HDF5ERROR, "Failed to select hyperslab");
+	}
     memspace = H5Screate_simple(rank, ext_dims, NULL);
 
     free(hdfdims);
@@ -1337,6 +1403,102 @@ int ismrmrd_read_image(const ISMRMRD_Dataset *dset, const char *varname,
     return ISMRMRD_NOERROR;
 }
 
+
+int ismrmrd_append_waveform(const ISMRMRD_Dataset *dset, const ISMRMRD_Waveform *wav) {
+    int status;
+    char *path;
+    hid_t datatype;
+    HDF5_Waveform hdf5wav[1];
+
+    if (dset==NULL) {
+        return ISMRMRD_PUSH_ERR(ISMRMRD_RUNTIMEERROR, "Dataset pointer should not be NULL.");
+    }
+    if (wav==NULL) {
+        return ISMRMRD_PUSH_ERR(ISMRMRD_RUNTIMEERROR, "Acquisition pointer should not be NULL.");
+    }
+
+    /* The path to the acqusition data */
+    path = make_path(dset, "waveforms");
+
+    /* The acquisition datatype */
+    datatype = get_hdf5type_waveform();
+
+    /* Create the HDF5 version of the acquisition */
+    hdf5wav[0].head = wav->head;
+    hdf5wav[0].data.len = wav->head.number_of_samples * wav->head.channels;
+    hdf5wav[0].data.p = wav->data;
+
+    /* Write it */
+    status = append_element(dset, path, hdf5wav, datatype, 0, NULL);
+    if (status != ISMRMRD_NOERROR) {
+        return ISMRMRD_PUSH_ERR(ISMRMRD_FILEERROR, "Failed to append acquisition.");
+    }
+
+    free(path);
+
+    /* Clean up */
+    status = H5Tclose(datatype);
+    if (status < 0) {
+        H5Ewalk2(H5E_DEFAULT, H5E_WALK_UPWARD, walk_hdf5_errors, NULL);
+        return ISMRMRD_PUSH_ERR(ISMRMRD_HDF5ERROR, "Failed to close datatype.");
+    }
+
+    return ISMRMRD_NOERROR;
+}
+
+int ismrmrd_read_waveform(const ISMRMRD_Dataset *dset, uint32_t index, ISMRMRD_Waveform *wav)
+{
+    hid_t datatype;
+    herr_t status;
+    HDF5_Waveform hdf5wav;
+    char *path;
+
+    if (dset==NULL) {
+        return ISMRMRD_PUSH_ERR(ISMRMRD_RUNTIMEERROR, "Dataset pointer should not be NULL.");
+    }
+    if (wav==NULL) {
+        return ISMRMRD_PUSH_ERR(ISMRMRD_RUNTIMEERROR, "Waveform pointer should not be NULL.");
+    }
+
+    /* The path to the acquisition data */
+    path = make_path(dset, "waveforms");
+
+    /* The acquisition datatype */
+    datatype = get_hdf5type_waveform();
+
+    status = read_element(dset, path, &hdf5wav, datatype, index);
+    memcpy(&wav->head, &hdf5wav.head, sizeof(ISMRMRD_WaveformHeader));
+    ismrmrd_make_consistent_waveform(wav);
+    memcpy(wav->data, hdf5wav.data.p, ismrmrd_size_of_waveform_data(wav));
+
+    /* clean up */
+    free(path);
+    free(hdf5wav.data.p);
+
+    status = H5Tclose(datatype);
+    if (status < 0) {
+        H5Ewalk2(H5E_DEFAULT, H5E_WALK_UPWARD, walk_hdf5_errors, NULL);
+        return ISMRMRD_PUSH_ERR(ISMRMRD_HDF5ERROR, "Failed to close datatype.");
+    }
+
+    return ISMRMRD_NOERROR;
+}
+
+uint32_t ismrmrd_get_number_of_waveforms(const ISMRMRD_Dataset *dset) {
+    char *path;
+    uint32_t numacq;
+
+    if (dset==NULL) {
+        ISMRMRD_PUSH_ERR(ISMRMRD_RUNTIMEERROR, "Pointer should not be NULL.");
+        return 0;
+    }
+    /* The path to the acqusition data */
+    path = make_path(dset, "waveforms");
+    numacq = get_number_of_elements(dset, path);
+    free(path);
+    return numacq;
+}
+
 int ismrmrd_append_array(const ISMRMRD_Dataset *dset, const char *varname, const ISMRMRD_NDArray *arr) {
     int status;
     hid_t datatype;
